@import url(http://fonts.googleapis.com/css?family=Roboto:400,400italic,500,500italic);

pre, tt {
    font-family:  monospace, sans-serif;
}

tt {
    font-size: 100%;
}

div.body h1, div.body h2, div.body h3, div.body h4, div.body h5, div.body h6 {
    border: 0;
    margin: 0;
    padding: 0.3em 0;
}
