/*
 * Copyright (C) 2011 Tatsuhiro Tsujikawa
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package net.sourceforge.argparse4j.helper;

/**
 * <p>
 * This interface abstracts the way of counting text width.
 * </p>
 * 
 * <p>
 * Since argparse4j is command line argument parser package and it is intended
 * to be used in terminal emulator and the terminal emulator in general uses
 * fixed width fonts, the meaning of "width" here is given text consumes how
 * many fixed character width. For string consisting only with ascii latin
 * characters, it will be the same as the number of characters in given text.
 * But for CJK characters are concerned this is not the case.
 * </p>
 */
public interface TextWidthCounter {

    /**
     * Counts given text consumes how many fixed character width.
     * 
     * @param text
     *            The text to inspect.
     * @return The computed value.
     */
    int width(String text);
}
